package de.mcs.utils.codecs;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Arrays;
import java.util.Random;

import junit.framework.TestCase;
import de.mcs.utils.StreamHelper;

public class TestBase64 extends TestCase {

    public void testBase64EncodeDecode() {
        Random rnd = new Random(System.currentTimeMillis());
        for (int i = 0; i < 20; i++) {
            int size = rnd.nextInt(128 * 1024);
            byte[] bindata = new byte[size];
            rnd.nextBytes(bindata);
            String base64String = Base64.encodeBytes(bindata);
            byte[] bindata2 = Base64.decode(base64String);
            assertTrue(Arrays.equals(bindata, bindata2));
        }
    }

    public void testBase64InputStream() throws IOException {
        Random rnd = new Random(System.currentTimeMillis());
        int size = rnd.nextInt(128 * 1024);
        byte[] bindata = new byte[size];
        rnd.nextBytes(bindata);
        File orgFile = File.createTempFile("tmp", "org");
        File b64File = File.createTempFile("tmp", "b64");
        File binFile = File.createTempFile("tmp", "bin");

        try {
            FileOutputStream fout = new FileOutputStream(orgFile);
            fout.write(bindata);
            fout.close();

            FileInputStream fin = new FileInputStream(orgFile);
            fout = new FileOutputStream(b64File);
            Base64.Base64InputStream b64In = new Base64.Base64InputStream(fin,
                    Base64.ENCODE);
            StreamHelper.copyStream(b64In, fout);
            b64In.close();
            fout.close();

            fin = new FileInputStream(b64File);
            fout = new FileOutputStream(binFile);
            b64In = new Base64.Base64InputStream(fin, Base64.DECODE);
            StreamHelper.copyStream(b64In, fout);
            b64In.close();
            fout.close();

            byte[] bindata2 = new byte[size];
            fin = new FileInputStream(binFile);
            fin.read(bindata2);
            fin.close();

            assertTrue(Arrays.equals(bindata, bindata2));
        } finally {
            orgFile.delete();
            b64File.delete();
            binFile.delete();
        }
    }

    public void testBase64OutputStream() throws IOException {
        Random rnd = new Random(System.currentTimeMillis());
        int size = rnd.nextInt(128 * 1024);
        byte[] bindata = new byte[size];
        rnd.nextBytes(bindata);
        File orgFile = File.createTempFile("tmp", "org");
        File b64File = File.createTempFile("tmp", "b64");
        File binFile = File.createTempFile("tmp", "bin");

        try {
            FileOutputStream fout = new FileOutputStream(orgFile);
            fout.write(bindata);
            fout.close();

            FileInputStream fin = new FileInputStream(orgFile);
            fout = new FileOutputStream(b64File);
            Base64.Base64OutputStream b64Out = new Base64.Base64OutputStream(fout,
                    Base64.ENCODE);
            StreamHelper.copyStream(fin, b64Out);
            b64Out.close();
            fout.close();

            fin = new FileInputStream(b64File);
            fout = new FileOutputStream(binFile);
            b64Out = new Base64.Base64OutputStream(fout, Base64.DECODE);
            StreamHelper.copyStream(fin, b64Out);
            b64Out.close();
            fout.close();

            byte[] bindata2 = new byte[size];
            fin = new FileInputStream(binFile);
            fin.read(bindata2);
            fin.close();

            assertTrue(Arrays.equals(bindata, bindata2));
        } finally {
            orgFile.delete();
            b64File.delete();
            binFile.delete();
        }
    }
}
